import numpy as np
import os

#k_same sequence mechanism, processes data from a particular stimulus/task
def k_same(X, k, ppts):
    # all rows of X represent feature vectors from ppts viewing this stimulus
    # cols of X are the feature dimensions
    # ppts is a 1d array that tracks the sub_id # for each row of X
    # k (int) is mechanism parameter
    unique_ppts  = np.unique(ppts)
    num_ppts     = len(unique_ppts)
    num_features = X.shape[1]

    #represent features as a 3D array, (X,Y,Z):
    #          X: rows, represent events/time windows
    #          Y: cols, represent features
    #          Z: depth, represent ppt id

    # compute t_min and t_max, in case input data has not been padded to have same length for all ppts
    t_min = np.inf
    t_max = 0
    for ppt in unique_ppts:
        ppt_idx = ppts == ppt
        num_idx = np.count_nonzero(ppt_idx)

        t_min = np.minimum(t_min, num_idx)
        t_max = np.maximum(t_max, num_idx)
    t_min = int(t_min)

    # declare a new array to fill NOTE: t_min is used for rows to be safe
    # , however input data should be padded such that t_min == t_max
    X_reshaped = np.zeros((t_min, num_features, num_ppts))
    #seperate 1d array to track sub_ids
    results_ppts = []

    counter = 0
    # loop over individuals adding them to new 3D array representation
    for ppt in unique_ppts:
        ppt_idx  = ppts == ppt
        ppt_data = X[ppt_idx, :]

        # add this along third dimension for each ppt
        X_reshaped[:, :, counter] = ppt_data[:t_min, :]

        # results_ppts is 1D array that contains all ground truth labels
        results_ppts = np.concatenate((results_ppts, np.array(ppts[ppt_idx][:t_min])))

        counter += 1

    # num centroids/groups: H
    H = int(np.ceil(num_ppts / k))

    # structure to hold k-anonymous results
    results = np.zeros_like(X_reshaped)

    # randomize indicies in range(0,num_ppts)
    rand_idx = np.arange(num_ppts)
    np.random.shuffle(rand_idx)

    # repeat ppts in groupings to fill all H groups in case num_ppts is not divisible by k
    num_repeats = H * k - num_ppts

    # reverse rand_idx, then append reversed[:num_repeats] to rand_idx to reach H * k elements
    rand_idx_reverse = np.flip(rand_idx)
    rand_idx = np.append(rand_idx, rand_idx_reverse[:num_repeats])

    #Random H groups of k elements using a 2d array with H rows and k cols, elements are sub_ids in each group (row)
    rand_idx = rand_idx.reshape((H, k))

    # loop over sequence (rows of X_reshaped) and fill with k copies of group centroids
    for t in range(0, t_min):
        curr_data = X_reshaped[t, :, :]

        out_data = np.zeros_like(curr_data)

        #for each of H groups average the k feature vectors to get each centroid, and put k copies of those into result
        for i in range(0, rand_idx.shape[0]):
            curr_idx   = rand_idx[i, :]
            curr_feats = curr_data[:, curr_idx]

            curr_centroid = np.mean(curr_feats, axis=1).reshape(-1, 1)

            out_data[:, curr_idx] = curr_centroid

        results[t, :, :] = out_data

    #remove third (ppt) dimension by stacking all ppt data vertically to match input format
    results_reshaped = results.swapaxes(0, 2).swapaxes(1, 2).reshape(-1, num_features)

    #return k-anon feature data and corresponding sub_id labels
    return results_reshaped, results_ppts

#save from dict format into feature .csv
def save_dict(out_dataset_path,feature,private_data_dict):
    #if doesn't exist, make it
    if not os.path.isdir(out_dataset_path):
        os.makedirs(out_dataset_path)


    #Loop through each img
    for img in private_data_dict.keys():
        curr_img_data = private_data_dict[img]

        #seperate out the sub_id col
        sub_ids       = curr_img_data[:,0]
        curr_img_data = curr_img_data[:,1:]

        #extract the unique subjects from with this dict data entry
        unique_sub_ids = np.unique(sub_ids)

        for sub_id in unique_sub_ids:
            sub_idx       = sub_ids == sub_id
            curr_sub_data = curr_img_data[sub_idx,:]

            out_file = 'Data_P' + str(int(sub_id)) + '_R' + str(img) + '_' + feature + '.csv'

            #write out
            np.savetxt(os.path.join(out_dataset_path,out_file),curr_sub_data,delimiter=",")